"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .message import Message, MessageTypedDict
from .responseformat import ResponseFormat, ResponseFormatTypedDict
from .tool import Tool, ToolTypedDict
from friendli.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypedDict


class DedicatedChatStreamBodyLogitBiasTypedDict(TypedDict):
    r"""Accepts a JSON object that maps tokens to an associated bias value. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model."""


class DedicatedChatStreamBodyLogitBias(BaseModel):
    r"""Accepts a JSON object that maps tokens to an associated bias value. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model."""


class DedicatedChatStreamBodyStreamOptionsTypedDict(TypedDict):
    r"""Options related to stream.
    It can only be used when `stream: true`.

    """

    include_usage: NotRequired[Nullable[bool]]
    r"""When set to `true`,
    the number of tokens used will be included at the end of the stream result in the form of
    `\"usage\": {\"completion_tokens\": number, \"prompt_tokens\": number, \"total_tokens\": number}`.

    """


class DedicatedChatStreamBodyStreamOptions(BaseModel):
    r"""Options related to stream.
    It can only be used when `stream: true`.

    """

    include_usage: OptionalNullable[bool] = UNSET
    r"""When set to `true`,
    the number of tokens used will be included at the end of the stream result in the form of
    `\"usage\": {\"completion_tokens\": number, \"prompt_tokens\": number, \"total_tokens\": number}`.

    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["include_usage"]
        nullable_fields = ["include_usage"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


DedicatedChatStreamBodyToolChoiceType = Literal["function"]
r"""The type of the tool. Currently, only `function` is supported."""


class DedicatedChatStreamBodyToolChoiceFunctionTypedDict(TypedDict):
    name: str
    r"""The name of the function to be called. Must be a-z, A-Z, 0-9, or contain underscores and dashes, with a maximum length of 64."""


class DedicatedChatStreamBodyToolChoiceFunction(BaseModel):
    name: str
    r"""The name of the function to be called. Must be a-z, A-Z, 0-9, or contain underscores and dashes, with a maximum length of 64."""


class DedicatedChatStreamBodyToolChoiceObjectTypedDict(TypedDict):
    type: DedicatedChatStreamBodyToolChoiceType
    r"""The type of the tool. Currently, only `function` is supported."""
    function: DedicatedChatStreamBodyToolChoiceFunctionTypedDict


class DedicatedChatStreamBodyToolChoiceObject(BaseModel):
    type: DedicatedChatStreamBodyToolChoiceType
    r"""The type of the tool. Currently, only `function` is supported."""

    function: DedicatedChatStreamBodyToolChoiceFunction


DedicatedChatStreamBodyToolChoiceTypedDict = Union[
    DedicatedChatStreamBodyToolChoiceObjectTypedDict, str
]
r"""Determines the tool calling behavior of the model.
When set to `none`, the model will bypass tool execution and generate a response directly.
In `auto` mode (the default), the model dynamically decides whether to call a tool or respond with a message.
Alternatively, setting `required` ensures that the model invokes at least one tool before responding to the user.
You can also specify a particular tool by `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}`.

"""


DedicatedChatStreamBodyToolChoice = Union[DedicatedChatStreamBodyToolChoiceObject, str]
r"""Determines the tool calling behavior of the model.
When set to `none`, the model will bypass tool execution and generate a response directly.
In `auto` mode (the default), the model dynamically decides whether to call a tool or respond with a message.
Alternatively, setting `required` ensures that the model invokes at least one tool before responding to the user.
You can also specify a particular tool by `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}`.

"""


class DedicatedChatStreamBodyTypedDict(TypedDict):
    model: str
    r"""ID of target endpoint. If you want to send request to specific adapter, using \"ENDPOINT_ID:ADAPTER_ROUTE\" format."""
    messages: List[MessageTypedDict]
    r"""A list of messages comprising the conversation so far."""
    eos_token: NotRequired[Nullable[List[int]]]
    r"""A list of endpoint sentence tokens."""
    frequency_penalty: NotRequired[Nullable[float]]
    r"""Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled, taking into account their frequency in the preceding text. This penalization diminishes the model's tendency to reproduce identical lines verbatim."""
    logit_bias: NotRequired[Nullable[DedicatedChatStreamBodyLogitBiasTypedDict]]
    r"""Accepts a JSON object that maps tokens to an associated bias value. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model."""
    logprobs: NotRequired[Nullable[bool]]
    r"""Whether to return log probabilities of the output tokens or not."""
    max_tokens: NotRequired[Nullable[int]]
    r"""The maximum number of tokens to generate. For decoder-only models like GPT, the length of your input tokens plus `max_tokens` should not exceed the model's maximum length (e.g., 2048 for OpenAI GPT-3). For encoder-decoder models like T5 or BlenderBot, `max_tokens` should not exceed the model's maximum output length. This is similar to Hugging Face's [`max_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.max_new_tokens) argument."""
    min_tokens: NotRequired[Nullable[int]]
    r"""The minimum number of tokens to generate. Default value is 0. This is similar to Hugging Face's [`min_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.min_new_tokens) argument.

    **This field is unsupported when `tools` are specified.**

    """
    n: NotRequired[Nullable[int]]
    r"""The number of independently generated results for the prompt. Not supported when using beam search. Defaults to 1. This is similar to Hugging Face's [`num_return_sequences`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.num_return_sequences) argument."""
    parallel_tool_calls: NotRequired[Nullable[bool]]
    r"""Whether to enable parallel function calling."""
    presence_penalty: NotRequired[Nullable[float]]
    r"""Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled at least once in the existing text."""
    repetition_penalty: NotRequired[Nullable[float]]
    r"""Penalizes tokens that have already appeared in the generated result (plus the input tokens for decoder-only models). Should be greater than or equal to 1.0 (1.0 means no penalty). See [keskar et al., 2019](https://arxiv.org/abs/1909.05858) for more details. This is similar to Hugging Face's [`repetition_penalty`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.repetition_penalty) argument."""
    response_format: NotRequired[Nullable[ResponseFormatTypedDict]]
    r"""The enforced format of the model's output.

    Note that the content of the output message may be truncated if it exceeds the `max_tokens`.
    You can check this by verifying that the `finish_reason` of the output message is `length`.

    ***Important***
    You must explicitly instruct the model to produce the desired output format using a system prompt or user message (e.g., `You are an API generating a valid JSON as output.`).
    Otherwise, the model may result in an unending stream of whitespace or other characters.

    """
    seed: NotRequired[Nullable[List[int]]]
    r"""Seed to control random procedure. If nothing is given, random seed is used for sampling, and return the seed along with the generated result. When using the `n` argument, you can pass a list of seed values to control all of the independent generations."""
    stop: NotRequired[Nullable[List[str]]]
    r"""When one of the stop phrases appears in the generation result, the API will stop generation. The stop phrases are excluded from the result. Defaults to empty list."""
    stream: NotRequired[Nullable[bool]]
    r"""Whether to stream generation result. When set true, each token will be sent as [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#event_stream_format) once generated."""
    stream_options: NotRequired[Nullable[DedicatedChatStreamBodyStreamOptionsTypedDict]]
    r"""Options related to stream.
    It can only be used when `stream: true`.

    """
    temperature: NotRequired[Nullable[float]]
    r"""Sampling temperature. Smaller temperature makes the generation result closer to greedy, argmax (i.e., `top_k = 1`) sampling. Defaults to 1.0. This is similar to Hugging Face's [`temperature`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.temperature) argument."""
    timeout_microseconds: NotRequired[Nullable[int]]
    r"""Request timeout. Gives the `HTTP 429 Too Many Requests` response status code. Default behavior is no timeout."""
    tool_choice: NotRequired[DedicatedChatStreamBodyToolChoiceTypedDict]
    r"""Determines the tool calling behavior of the model.
    When set to `none`, the model will bypass tool execution and generate a response directly.
    In `auto` mode (the default), the model dynamically decides whether to call a tool or respond with a message.
    Alternatively, setting `required` ensures that the model invokes at least one tool before responding to the user.
    You can also specify a particular tool by `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}`.

    """
    tools: NotRequired[Nullable[List[ToolTypedDict]]]
    r"""A list of tools the model may call.
    Currently, only functions are supported as a tool.
    A maximum of 128 functions is supported.
    Use this to provide a list of functions the model may generate JSON inputs for.

    **When `tools` are specified, `min_tokens` field is unsupported.**

    """
    top_k: NotRequired[Nullable[int]]
    r"""The number of highest probability tokens to keep for sampling. Numbers between 0 and the vocab size of the model (both inclusive) are allowed. The default value is 0, which means that the API does not apply top-k filtering. This is similar to Hugging Face's [`top_k`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_k) argument."""
    top_logprobs: NotRequired[Nullable[int]]
    r"""The number of most likely tokens to return at each token position, each with an associated log probability. `logprobs` must be set to true if this parameter is used."""
    top_p: NotRequired[Nullable[float]]
    r"""Tokens comprising the top `top_p` probability mass are kept for sampling. Numbers between 0.0 (exclusive) and 1.0 (inclusive) are allowed. Defaults to 1.0. This is similar to Hugging Face's [`top_p`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_p) argument."""


class DedicatedChatStreamBody(BaseModel):
    model: str
    r"""ID of target endpoint. If you want to send request to specific adapter, using \"ENDPOINT_ID:ADAPTER_ROUTE\" format."""

    messages: List[Message]
    r"""A list of messages comprising the conversation so far."""

    eos_token: OptionalNullable[List[int]] = UNSET
    r"""A list of endpoint sentence tokens."""

    frequency_penalty: OptionalNullable[float] = UNSET
    r"""Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled, taking into account their frequency in the preceding text. This penalization diminishes the model's tendency to reproduce identical lines verbatim."""

    logit_bias: OptionalNullable[DedicatedChatStreamBodyLogitBias] = UNSET
    r"""Accepts a JSON object that maps tokens to an associated bias value. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model."""

    logprobs: OptionalNullable[bool] = UNSET
    r"""Whether to return log probabilities of the output tokens or not."""

    max_tokens: OptionalNullable[int] = UNSET
    r"""The maximum number of tokens to generate. For decoder-only models like GPT, the length of your input tokens plus `max_tokens` should not exceed the model's maximum length (e.g., 2048 for OpenAI GPT-3). For encoder-decoder models like T5 or BlenderBot, `max_tokens` should not exceed the model's maximum output length. This is similar to Hugging Face's [`max_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.max_new_tokens) argument."""

    min_tokens: OptionalNullable[int] = 0
    r"""The minimum number of tokens to generate. Default value is 0. This is similar to Hugging Face's [`min_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.min_new_tokens) argument.

    **This field is unsupported when `tools` are specified.**

    """

    n: OptionalNullable[int] = 1
    r"""The number of independently generated results for the prompt. Not supported when using beam search. Defaults to 1. This is similar to Hugging Face's [`num_return_sequences`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.num_return_sequences) argument."""

    parallel_tool_calls: OptionalNullable[bool] = UNSET
    r"""Whether to enable parallel function calling."""

    presence_penalty: OptionalNullable[float] = UNSET
    r"""Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled at least once in the existing text."""

    repetition_penalty: OptionalNullable[float] = UNSET
    r"""Penalizes tokens that have already appeared in the generated result (plus the input tokens for decoder-only models). Should be greater than or equal to 1.0 (1.0 means no penalty). See [keskar et al., 2019](https://arxiv.org/abs/1909.05858) for more details. This is similar to Hugging Face's [`repetition_penalty`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.repetition_penalty) argument."""

    response_format: OptionalNullable[ResponseFormat] = UNSET
    r"""The enforced format of the model's output.

    Note that the content of the output message may be truncated if it exceeds the `max_tokens`.
    You can check this by verifying that the `finish_reason` of the output message is `length`.

    ***Important***
    You must explicitly instruct the model to produce the desired output format using a system prompt or user message (e.g., `You are an API generating a valid JSON as output.`).
    Otherwise, the model may result in an unending stream of whitespace or other characters.

    """

    seed: OptionalNullable[List[int]] = UNSET
    r"""Seed to control random procedure. If nothing is given, random seed is used for sampling, and return the seed along with the generated result. When using the `n` argument, you can pass a list of seed values to control all of the independent generations."""

    stop: OptionalNullable[List[str]] = UNSET
    r"""When one of the stop phrases appears in the generation result, the API will stop generation. The stop phrases are excluded from the result. Defaults to empty list."""

    stream: OptionalNullable[bool] = True
    r"""Whether to stream generation result. When set true, each token will be sent as [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#event_stream_format) once generated."""

    stream_options: OptionalNullable[DedicatedChatStreamBodyStreamOptions] = UNSET
    r"""Options related to stream.
    It can only be used when `stream: true`.

    """

    temperature: OptionalNullable[float] = 1
    r"""Sampling temperature. Smaller temperature makes the generation result closer to greedy, argmax (i.e., `top_k = 1`) sampling. Defaults to 1.0. This is similar to Hugging Face's [`temperature`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.temperature) argument."""

    timeout_microseconds: OptionalNullable[int] = UNSET
    r"""Request timeout. Gives the `HTTP 429 Too Many Requests` response status code. Default behavior is no timeout."""

    tool_choice: Optional[DedicatedChatStreamBodyToolChoice] = None
    r"""Determines the tool calling behavior of the model.
    When set to `none`, the model will bypass tool execution and generate a response directly.
    In `auto` mode (the default), the model dynamically decides whether to call a tool or respond with a message.
    Alternatively, setting `required` ensures that the model invokes at least one tool before responding to the user.
    You can also specify a particular tool by `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}`.

    """

    tools: OptionalNullable[List[Tool]] = UNSET
    r"""A list of tools the model may call.
    Currently, only functions are supported as a tool.
    A maximum of 128 functions is supported.
    Use this to provide a list of functions the model may generate JSON inputs for.

    **When `tools` are specified, `min_tokens` field is unsupported.**

    """

    top_k: OptionalNullable[int] = 0
    r"""The number of highest probability tokens to keep for sampling. Numbers between 0 and the vocab size of the model (both inclusive) are allowed. The default value is 0, which means that the API does not apply top-k filtering. This is similar to Hugging Face's [`top_k`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_k) argument."""

    top_logprobs: OptionalNullable[int] = UNSET
    r"""The number of most likely tokens to return at each token position, each with an associated log probability. `logprobs` must be set to true if this parameter is used."""

    top_p: OptionalNullable[float] = 1
    r"""Tokens comprising the top `top_p` probability mass are kept for sampling. Numbers between 0.0 (exclusive) and 1.0 (inclusive) are allowed. Defaults to 1.0. This is similar to Hugging Face's [`top_p`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_p) argument."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "eos_token",
            "frequency_penalty",
            "logit_bias",
            "logprobs",
            "max_tokens",
            "min_tokens",
            "n",
            "parallel_tool_calls",
            "presence_penalty",
            "repetition_penalty",
            "response_format",
            "seed",
            "stop",
            "stream",
            "stream_options",
            "temperature",
            "timeout_microseconds",
            "tool_choice",
            "tools",
            "top_k",
            "top_logprobs",
            "top_p",
        ]
        nullable_fields = [
            "eos_token",
            "frequency_penalty",
            "logit_bias",
            "logprobs",
            "max_tokens",
            "min_tokens",
            "n",
            "parallel_tool_calls",
            "presence_penalty",
            "repetition_penalty",
            "response_format",
            "seed",
            "stop",
            "stream",
            "stream_options",
            "temperature",
            "timeout_microseconds",
            "tools",
            "top_k",
            "top_logprobs",
            "top_p",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



from .sdk import Friendli
from .httpclient import AsyncHttpClient, HttpClient
from .errorhttpxclients import ErrorThrowingSyncClient, ErrorThrowingAsyncClient
from .utils.logger import Logger
from .utils.retries import RetryConfig
from friendli.types import OptionalNullable, UNSET
import httpx
from typing import Callable, Dict, Optional, Type, Union
from types import TracebackType


class SyncFriendli(Friendli):
    def __init__(
        self,
        token: Optional[Union[Optional[str], Callable[[], Optional[str]]]] = None,
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        super().__init__(
            token=token,
            server_idx=server_idx,
            server_url=server_url,
            url_params=url_params,
            client=client or httpx.Client(),
            async_client=ErrorThrowingAsyncClient(),
            retry_config=retry_config,
            timeout_ms=timeout_ms,
            debug_logger=debug_logger,
        )

    def __enter__(self):
        return self

    def __exit__(
        self,
        exc_type: Union[Type[BaseException], None],
        exc_val: Union[BaseException, None],
        exc_tb: Union[TracebackType, None]
    ) -> None:
        self.sdk_configuration.client.__exit__(exc_type, exc_val, exc_tb)


class AsyncFriendli(Friendli):
    def __init__(
        self,
        token: Optional[Union[Optional[str], Callable[[], Optional[str]]]] = None,
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        super().__init__(
            token=token,
            server_idx=server_idx,
            server_url=server_url,
            url_params=url_params,
            client=ErrorThrowingSyncClient(),
            async_client=async_client or httpx.AsyncClient(),
            retry_config=retry_config,
            timeout_ms=timeout_ms,
            debug_logger=debug_logger,
        )

    async def __aenter__(self):
        return self

    async def __aexit__(
        self,
        exc_type: Union[Type[BaseException], None],
        exc_val: Union[BaseException, None],
        exc_tb: Union[TracebackType, None]
    ) -> None:
        await self.sdk_configuration.async_client.__aexit__(exc_type, exc_val, exc_tb)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .streamedcompletionstokencomplete import (
    StreamedCompletionsTokenComplete,
    StreamedCompletionsTokenCompleteTypedDict,
)
from .streamedcompletionstokensampled import (
    StreamedCompletionsTokenSampled,
    StreamedCompletionsTokenSampledTypedDict,
)
from friendli.types import BaseModel
from friendli.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypedDict


StreamedCompletionsResultDataTypedDict = Union[
    StreamedCompletionsTokenCompleteTypedDict, StreamedCompletionsTokenSampledTypedDict
]


StreamedCompletionsResultData = Annotated[
    Union[
        Annotated[StreamedCompletionsTokenSampled, Tag("token_sampled")],
        Annotated[StreamedCompletionsTokenComplete, Tag("complete")],
    ],
    Discriminator(lambda m: get_discriminator(m, "event", "event")),
]


class StreamedCompletionsResultTypedDict(TypedDict):
    r"""Successfully generated completions."""

    data: StreamedCompletionsResultDataTypedDict


class StreamedCompletionsResult(BaseModel):
    r"""Successfully generated completions."""

    data: StreamedCompletionsResultData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .completionschoice import CompletionsChoice, CompletionsChoiceTypedDict
from .usage import Usage, UsageTypedDict
from friendli.types import BaseModel
from typing import List, Literal
from typing_extensions import TypedDict


StreamedCompletionsTokenCompleteEvent = Literal["complete"]
r"""Type of server-sent event."""


class StreamedCompletionsTokenCompleteTypedDict(TypedDict):
    event: StreamedCompletionsTokenCompleteEvent
    r"""Type of server-sent event."""
    choices: List[CompletionsChoiceTypedDict]
    usage: UsageTypedDict


class StreamedCompletionsTokenComplete(BaseModel):
    event: StreamedCompletionsTokenCompleteEvent
    r"""Type of server-sent event."""

    choices: List[CompletionsChoice]

    usage: Usage



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .completionschoice import CompletionsChoice, CompletionsChoiceTypedDict
from .usage import Usage, UsageTypedDict
from friendli.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class CompletionsResultTypedDict(TypedDict):
    r"""Successfully generated completions."""

    choices: List[CompletionsChoiceTypedDict]
    usage: UsageTypedDict


class CompletionsResult(BaseModel):
    r"""Successfully generated completions."""

    choices: List[CompletionsChoice]

    usage: Usage



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatchoice import ChatChoice, ChatChoiceTypedDict
from .usage import Usage, UsageTypedDict
from friendli.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ChatResultTypedDict(TypedDict):
    r"""Successfully generated a chat response."""

    choices: List[ChatChoiceTypedDict]
    usage: UsageTypedDict
    created: NotRequired[int]
    r"""The Unix timestamp (in seconds) for when the generation completed."""


class ChatResult(BaseModel):
    r"""Successfully generated a chat response."""

    choices: List[ChatChoice]

    usage: Usage

    created: Optional[int] = None
    r"""The Unix timestamp (in seconds) for when the generation completed."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class TokenizationResultTypedDict(TypedDict):
    r"""Successfully tokenized the text."""

    tokens: NotRequired[List[int]]
    r"""A list of token IDs."""


class TokenizationResult(BaseModel):
    r"""Successfully tokenized the text."""

    tokens: Optional[List[int]] = None
    r"""A list of token IDs."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class TopLogprobsTypedDict(TypedDict):
    token: NotRequired[str]
    r"""The token."""
    logprob: NotRequired[float]
    r"""The log probability of this token."""


class TopLogprobs(BaseModel):
    token: Optional[str] = None
    r"""The token."""

    logprob: Optional[float] = None
    r"""The log probability of this token."""


class ContentTypedDict(TypedDict):
    token: NotRequired[str]
    r"""The token."""
    logprob: NotRequired[float]
    r"""The log probability of this token."""
    top_logprobs: NotRequired[List[TopLogprobsTypedDict]]
    r"""List of the most likely tokens and their log probability, at this token position."""


class Content(BaseModel):
    token: Optional[str] = None
    r"""The token."""

    logprob: Optional[float] = None
    r"""The log probability of this token."""

    top_logprobs: Optional[List[TopLogprobs]] = None
    r"""List of the most likely tokens and their log probability, at this token position."""


class LogprobsTypedDict(TypedDict):
    r"""Log probability information for the choice."""

    content: NotRequired[List[ContentTypedDict]]
    r"""A list of message content tokens with log probability information."""


class Logprobs(BaseModel):
    r"""Log probability information for the choice."""

    content: Optional[List[Content]] = None
    r"""A list of message content tokens with log probability information."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import List, Literal
from typing_extensions import TypedDict


RagBuiltInToolType = Literal["rag:knowledge-base"]
r"""The type of the rag knowledge-base tool."""


class RagBuiltInToolTypedDict(TypedDict):
    type: RagBuiltInToolType
    r"""The type of the rag knowledge-base tool."""
    knowledge_base_ids: List[str]
    r"""A List of knowledge-base IDs. For now, only one knowledge-base is supported."""


class RagBuiltInTool(BaseModel):
    type: RagBuiltInToolType
    r"""The type of the rag knowledge-base tool."""

    knowledge_base_ids: List[str]
    r"""A List of knowledge-base IDs. For now, only one knowledge-base is supported."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from friendli import models, utils
from friendli._hooks import HookContext
from friendli.types import OptionalNullable, UNSET
from friendli.utils import eventstreaming, get_security_from_env
from typing import AsyncGenerator, Generator, Optional, Union


class FriendliCompletions(BaseSDK):
    def complete(
        self,
        *,
        dedicated_completions_complete_body: Union[
            models.DedicatedCompletionsCompleteBody,
            models.DedicatedCompletionsCompleteBodyTypedDict,
        ],
        x_friendli_team: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> models.CompletionsResult:
        r"""Completions

        Generate text based on the given text prompt.

        :param dedicated_completions_complete_body:
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.DedicatedCompletionsCompleteRequest(
            x_friendli_team=x_friendli_team,
            dedicated_completions_complete_body=utils.get_pydantic_model(
                dedicated_completions_complete_body,
                models.DedicatedCompletionsCompleteBody,
            ),
        )

        req = self.build_request(
            method="POST",
            path="/dedicated/v1/completions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.dedicated_completions_complete_body,
                False,
                False,
                "json",
                models.DedicatedCompletionsCompleteBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="dedicatedCompletionsComplete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.CompletionsResult)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def complete_async(
        self,
        *,
        dedicated_completions_complete_body: Union[
            models.DedicatedCompletionsCompleteBody,
            models.DedicatedCompletionsCompleteBodyTypedDict,
        ],
        x_friendli_team: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> models.CompletionsResult:
        r"""Completions

        Generate text based on the given text prompt.

        :param dedicated_completions_complete_body:
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.DedicatedCompletionsCompleteRequest(
            x_friendli_team=x_friendli_team,
            dedicated_completions_complete_body=utils.get_pydantic_model(
                dedicated_completions_complete_body,
                models.DedicatedCompletionsCompleteBody,
            ),
        )

        req = self.build_request_async(
            method="POST",
            path="/dedicated/v1/completions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.dedicated_completions_complete_body,
                False,
                False,
                "json",
                models.DedicatedCompletionsCompleteBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="dedicatedCompletionsComplete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.CompletionsResult)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def stream(
        self,
        *,
        dedicated_completions_stream_body: Union[
            models.DedicatedCompletionsStreamBody,
            models.DedicatedCompletionsStreamBodyTypedDict,
        ],
        x_friendli_team: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Generator[models.StreamedCompletionsResult, None, None]:
        r"""Stream completions

        Generate text based on the given text prompt.

        :param dedicated_completions_stream_body:
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.DedicatedCompletionsStreamRequest(
            x_friendli_team=x_friendli_team,
            dedicated_completions_stream_body=utils.get_pydantic_model(
                dedicated_completions_stream_body, models.DedicatedCompletionsStreamBody
            ),
        )

        req = self.build_request(
            method="POST",
            path="/dedicated/v1/completions#stream",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.dedicated_completions_stream_body,
                False,
                False,
                "json",
                models.DedicatedCompletionsStreamBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="dedicatedCompletionsStream",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.stream_events(
                http_res,
                lambda raw: utils.unmarshal_json(raw, models.StreamedCompletionsResult),
                sentinel="[DONE]",
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def stream_async(
        self,
        *,
        dedicated_completions_stream_body: Union[
            models.DedicatedCompletionsStreamBody,
            models.DedicatedCompletionsStreamBodyTypedDict,
        ],
        x_friendli_team: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> AsyncGenerator[models.StreamedCompletionsResult, None]:
        r"""Stream completions

        Generate text based on the given text prompt.

        :param dedicated_completions_stream_body:
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.DedicatedCompletionsStreamRequest(
            x_friendli_team=x_friendli_team,
            dedicated_completions_stream_body=utils.get_pydantic_model(
                dedicated_completions_stream_body, models.DedicatedCompletionsStreamBody
            ),
        )

        req = self.build_request_async(
            method="POST",
            path="/dedicated/v1/completions#stream",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.dedicated_completions_stream_body,
                False,
                False,
                "json",
                models.DedicatedCompletionsStreamBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="dedicatedCompletionsStream",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.stream_events_async(
                http_res,
                lambda raw: utils.unmarshal_json(raw, models.StreamedCompletionsResult),
                sentinel="[DONE]",
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


Event = Literal["token_sampled"]
r"""Type of server-sent event."""


class StreamedCompletionsTokenSampledTypedDict(TypedDict):
    event: Event
    r"""Type of server-sent event."""
    index: int
    r"""The index of the choice in the list of generated choices."""
    text: str
    r"""Generated text output."""
    token: int
    r"""Generated output token."""


class StreamedCompletionsTokenSampled(BaseModel):
    event: Event
    r"""Type of server-sent event."""

    index: int
    r"""The index of the choice in the list of generated choices."""

    text: str
    r"""Generated text output."""

    token: int
    r"""Generated output token."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing_extensions import TypedDict


class DedicatedTokenizationBodyTypedDict(TypedDict):
    model: str
    r"""ID of target endpoint. If you want to send request to specific adapter, using \"ENDPOINT_ID:ADAPTER_ROUTE\" format."""
    prompt: str
    r"""Input text prompt to tokenize."""


class DedicatedTokenizationBody(BaseModel):
    model: str
    r"""ID of target endpoint. If you want to send request to specific adapter, using \"ENDPOINT_ID:ADAPTER_ROUTE\" format."""

    prompt: str
    r"""Input text prompt to tokenize."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .filebuiltintool import FileBuiltInTool, FileBuiltInToolTypedDict
from .functiontool import FunctionTool, FunctionToolTypedDict
from .otherbuiltintool import OtherBuiltInTool, OtherBuiltInToolTypedDict
from .ragbuiltintool import RagBuiltInTool, RagBuiltInToolTypedDict
from friendli.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated


ToolAssistedChatToolTypedDict = Union[
    OtherBuiltInToolTypedDict,
    FunctionToolTypedDict,
    FileBuiltInToolTypedDict,
    RagBuiltInToolTypedDict,
]


ToolAssistedChatTool = Annotated[
    Union[
        Annotated[FunctionTool, Tag("function")],
        Annotated[FileBuiltInTool, Tag("file:text")],
        Annotated[OtherBuiltInTool, Tag("math:calculator")],
        Annotated[OtherBuiltInTool, Tag("math:statistics")],
        Annotated[OtherBuiltInTool, Tag("math:calendar")],
        Annotated[OtherBuiltInTool, Tag("web:search")],
        Annotated[OtherBuiltInTool, Tag("web:url")],
        Annotated[OtherBuiltInTool, Tag("code:python-interpreter")],
        Annotated[RagBuiltInTool, Tag("rag:knowledge-base")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from friendli import models, utils
from friendli._hooks import HookContext
from friendli.types import OptionalNullable, UNSET
from friendli.utils import eventstreaming, get_security_from_env
from typing import AsyncGenerator, Generator, List, Optional, Union


class ToolAssistedChat(BaseSDK):
    def complete(
        self,
        *,
        model: str,
        messages: Union[List[models.Message], List[models.MessageTypedDict]],
        x_friendli_team: Optional[str] = None,
        eos_token: OptionalNullable[List[int]] = UNSET,
        frequency_penalty: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[int] = UNSET,
        min_tokens: OptionalNullable[int] = 0,
        n: OptionalNullable[int] = 1,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        presence_penalty: OptionalNullable[float] = UNSET,
        repetition_penalty: OptionalNullable[float] = UNSET,
        response_format: OptionalNullable[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = UNSET,
        resume_generation: OptionalNullable[bool] = UNSET,
        seed: OptionalNullable[List[int]] = UNSET,
        stop: OptionalNullable[List[str]] = UNSET,
        stream: OptionalNullable[bool] = False,
        temperature: OptionalNullable[float] = 1,
        timeout_microseconds: OptionalNullable[int] = UNSET,
        tool_choice: Optional[
            Union[
                models.ServerlessToolAssistedChatCompleteBodyToolChoice,
                models.ServerlessToolAssistedChatCompleteBodyToolChoiceTypedDict,
            ]
        ] = None,
        tools: OptionalNullable[
            Union[
                List[models.ToolAssistedChatTool],
                List[models.ToolAssistedChatToolTypedDict],
            ]
        ] = UNSET,
        top_k: OptionalNullable[int] = 0,
        top_p: OptionalNullable[float] = 1,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> models.ChatResult:
        r"""Tool assisted chat completions

        Given a list of messages forming a conversation, the model generates a response. Additionally, the model can utilize built-in tools for tool calls, enhancing its capability to provide more comprehensive and actionable responses.

        :param model: Code of the model to use. See [available model list](https://friendli.ai/docs/guides/serverless_endpoints/pricing#text-generation-models).
        :param messages: A list of messages comprising the conversation so far.
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param eos_token: A list of endpoint sentence tokens.
        :param frequency_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled, taking into account their frequency in the preceding text. This penalization diminishes the model's tendency to reproduce identical lines verbatim.
        :param max_tokens: The maximum number of tokens to generate. For decoder-only models like GPT, the length of your input tokens plus `max_tokens` should not exceed the model's maximum length (e.g., 2048 for OpenAI GPT-3). For encoder-decoder models like T5 or BlenderBot, `max_tokens` should not exceed the model's maximum output length. This is similar to Hugging Face's [`max_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.max_new_tokens) argument.
        :param min_tokens: The minimum number of tokens to generate. Default value is 0. This is similar to Hugging Face's [`min_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.min_new_tokens) argument.  **This field is unsupported when `tools` are specified.**
        :param n: The number of independently generated results for the prompt. Not supported when using beam search. Defaults to 1. This is similar to Hugging Face's [`num_return_sequences`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.num_return_sequences) argument.
        :param parallel_tool_calls: Whether to enable parallel function calling.
        :param presence_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled at least once in the existing text.
        :param repetition_penalty: Penalizes tokens that have already appeared in the generated result (plus the input tokens for decoder-only models). Should be greater than or equal to 1.0 (1.0 means no penalty). See [keskar et al., 2019](https://arxiv.org/abs/1909.05858) for more details. This is similar to Hugging Face's [`repetition_penalty`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.repetition_penalty) argument.
        :param response_format: The enforced format of the model's output.  Note that the content of the output message may be truncated if it exceeds the `max_tokens`. You can check this by verifying that the `finish_reason` of the output message is `length`.  ***Important*** You must explicitly instruct the model to produce the desired output format using a system prompt or user message (e.g., `You are an API generating a valid JSON as output.`). Otherwise, the model may result in an unending stream of whitespace or other characters.
        :param resume_generation: Enable to continue text generation even after an error occurs during a tool call.  Note that enabling this option may use more tokens, as the system generates additional content to handle errors gracefully. However, if the system fails more than 8 times, the generation will stop regardless.  ***Tip*** This is useful in scenarios where you want to maintain text generation flow despite errors, such as when generating long-form content. The user will not be interrupted by tool call issues, ensuring a smoother experience.
        :param seed: Seed to control random procedure. If nothing is given, random seed is used for sampling, and return the seed along with the generated result. When using the `n` argument, you can pass a list of seed values to control all of the independent generations.
        :param stop: When one of the stop phrases appears in the generation result, the API will stop generation. The stop phrases are excluded from the result. Defaults to empty list.
        :param stream: Whether to stream generation result. When set true, each token will be sent as [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#event_stream_format) once generated.
        :param temperature: Sampling temperature. Smaller temperature makes the generation result closer to greedy, argmax (i.e., `top_k = 1`) sampling. Defaults to 1.0. This is similar to Hugging Face's [`temperature`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.temperature) argument.
        :param timeout_microseconds: Request timeout. Gives the `HTTP 429 Too Many Requests` response status code. Default behavior is no timeout.
        :param tool_choice: Determines the tool calling behavior of the model. When set to `none`, the model will bypass tool execution and generate a response directly. In `auto` mode (the default), the model dynamically decides whether to call a tool or respond with a message. Alternatively, setting `required` ensures that the model invokes at least one tool before responding to the user. You can also specify a particular tool by `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}`.
        :param tools: A list of tools the model may call. A maximum of 128 functions is supported. Use this to provide a list of functions the model may generate JSON inputs for. For more detailed information about each tool, please refer [here](https://friendli.ai/docs/guides/serverless_endpoints/tools/built_in_tools).  **When `tools` are specified, `min_tokens` field is unsupported.**
        :param top_k: The number of highest probability tokens to keep for sampling. Numbers between 0 and the vocab size of the model (both inclusive) are allowed. The default value is 0, which means that the API does not apply top-k filtering. This is similar to Hugging Face's [`top_k`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_k) argument.
        :param top_p: Tokens comprising the top `top_p` probability mass are kept for sampling. Numbers between 0.0 (exclusive) and 1.0 (inclusive) are allowed. Defaults to 1.0. This is similar to Hugging Face's [`top_p`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_p) argument.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.ServerlessToolAssistedChatCompleteRequest(
            x_friendli_team=x_friendli_team,
            serverless_tool_assisted_chat_complete_body=models.ServerlessToolAssistedChatCompleteBody(
                model=model,
                messages=utils.get_pydantic_model(messages, List[models.Message]),
                eos_token=eos_token,
                frequency_penalty=frequency_penalty,
                max_tokens=max_tokens,
                min_tokens=min_tokens,
                n=n,
                parallel_tool_calls=parallel_tool_calls,
                presence_penalty=presence_penalty,
                repetition_penalty=repetition_penalty,
                response_format=utils.get_pydantic_model(
                    response_format, OptionalNullable[models.ResponseFormat]
                ),
                resume_generation=resume_generation,
                seed=seed,
                stop=stop,
                stream=stream,
                temperature=temperature,
                timeout_microseconds=timeout_microseconds,
                tool_choice=utils.get_pydantic_model(
                    tool_choice,
                    Optional[models.ServerlessToolAssistedChatCompleteBodyToolChoice],
                ),
                tools=utils.get_pydantic_model(
                    tools, OptionalNullable[List[models.ToolAssistedChatTool]]
                ),
                top_k=top_k,
                top_p=top_p,
            ),
        )

        req = self.build_request(
            method="POST",
            path="/serverless/tools/v1/chat/completions#complete",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.serverless_tool_assisted_chat_complete_body,
                False,
                False,
                "json",
                models.ServerlessToolAssistedChatCompleteBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="serverlessToolAssistedChatComplete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.ChatResult)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def complete_async(
        self,
        *,
        model: str,
        messages: Union[List[models.Message], List[models.MessageTypedDict]],
        x_friendli_team: Optional[str] = None,
        eos_token: OptionalNullable[List[int]] = UNSET,
        frequency_penalty: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[int] = UNSET,
        min_tokens: OptionalNullable[int] = 0,
        n: OptionalNullable[int] = 1,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        presence_penalty: OptionalNullable[float] = UNSET,
        repetition_penalty: OptionalNullable[float] = UNSET,
        response_format: OptionalNullable[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = UNSET,
        resume_generation: OptionalNullable[bool] = UNSET,
        seed: OptionalNullable[List[int]] = UNSET,
        stop: OptionalNullable[List[str]] = UNSET,
        stream: OptionalNullable[bool] = False,
        temperature: OptionalNullable[float] = 1,
        timeout_microseconds: OptionalNullable[int] = UNSET,
        tool_choice: Optional[
            Union[
                models.ServerlessToolAssistedChatCompleteBodyToolChoice,
                models.ServerlessToolAssistedChatCompleteBodyToolChoiceTypedDict,
            ]
        ] = None,
        tools: OptionalNullable[
            Union[
                List[models.ToolAssistedChatTool],
                List[models.ToolAssistedChatToolTypedDict],
            ]
        ] = UNSET,
        top_k: OptionalNullable[int] = 0,
        top_p: OptionalNullable[float] = 1,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> models.ChatResult:
        r"""Tool assisted chat completions

        Given a list of messages forming a conversation, the model generates a response. Additionally, the model can utilize built-in tools for tool calls, enhancing its capability to provide more comprehensive and actionable responses.

        :param model: Code of the model to use. See [available model list](https://friendli.ai/docs/guides/serverless_endpoints/pricing#text-generation-models).
        :param messages: A list of messages comprising the conversation so far.
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param eos_token: A list of endpoint sentence tokens.
        :param frequency_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled, taking into account their frequency in the preceding text. This penalization diminishes the model's tendency to reproduce identical lines verbatim.
        :param max_tokens: The maximum number of tokens to generate. For decoder-only models like GPT, the length of your input tokens plus `max_tokens` should not exceed the model's maximum length (e.g., 2048 for OpenAI GPT-3). For encoder-decoder models like T5 or BlenderBot, `max_tokens` should not exceed the model's maximum output length. This is similar to Hugging Face's [`max_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.max_new_tokens) argument.
        :param min_tokens: The minimum number of tokens to generate. Default value is 0. This is similar to Hugging Face's [`min_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.min_new_tokens) argument.  **This field is unsupported when `tools` are specified.**
        :param n: The number of independently generated results for the prompt. Not supported when using beam search. Defaults to 1. This is similar to Hugging Face's [`num_return_sequences`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.num_return_sequences) argument.
        :param parallel_tool_calls: Whether to enable parallel function calling.
        :param presence_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled at least once in the existing text.
        :param repetition_penalty: Penalizes tokens that have already appeared in the generated result (plus the input tokens for decoder-only models). Should be greater than or equal to 1.0 (1.0 means no penalty). See [keskar et al., 2019](https://arxiv.org/abs/1909.05858) for more details. This is similar to Hugging Face's [`repetition_penalty`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.repetition_penalty) argument.
        :param response_format: The enforced format of the model's output.  Note that the content of the output message may be truncated if it exceeds the `max_tokens`. You can check this by verifying that the `finish_reason` of the output message is `length`.  ***Important*** You must explicitly instruct the model to produce the desired output format using a system prompt or user message (e.g., `You are an API generating a valid JSON as output.`). Otherwise, the model may result in an unending stream of whitespace or other characters.
        :param resume_generation: Enable to continue text generation even after an error occurs during a tool call.  Note that enabling this option may use more tokens, as the system generates additional content to handle errors gracefully. However, if the system fails more than 8 times, the generation will stop regardless.  ***Tip*** This is useful in scenarios where you want to maintain text generation flow despite errors, such as when generating long-form content. The user will not be interrupted by tool call issues, ensuring a smoother experience.
        :param seed: Seed to control random procedure. If nothing is given, random seed is used for sampling, and return the seed along with the generated result. When using the `n` argument, you can pass a list of seed values to control all of the independent generations.
        :param stop: When one of the stop phrases appears in the generation result, the API will stop generation. The stop phrases are excluded from the result. Defaults to empty list.
        :param stream: Whether to stream generation result. When set true, each token will be sent as [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#event_stream_format) once generated.
        :param temperature: Sampling temperature. Smaller temperature makes the generation result closer to greedy, argmax (i.e., `top_k = 1`) sampling. Defaults to 1.0. This is similar to Hugging Face's [`temperature`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.temperature) argument.
        :param timeout_microseconds: Request timeout. Gives the `HTTP 429 Too Many Requests` response status code. Default behavior is no timeout.
        :param tool_choice: Determines the tool calling behavior of the model. When set to `none`, the model will bypass tool execution and generate a response directly. In `auto` mode (the default), the model dynamically decides whether to call a tool or respond with a message. Alternatively, setting `required` ensures that the model invokes at least one tool before responding to the user. You can also specify a particular tool by `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}`.
        :param tools: A list of tools the model may call. A maximum of 128 functions is supported. Use this to provide a list of functions the model may generate JSON inputs for. For more detailed information about each tool, please refer [here](https://friendli.ai/docs/guides/serverless_endpoints/tools/built_in_tools).  **When `tools` are specified, `min_tokens` field is unsupported.**
        :param top_k: The number of highest probability tokens to keep for sampling. Numbers between 0 and the vocab size of the model (both inclusive) are allowed. The default value is 0, which means that the API does not apply top-k filtering. This is similar to Hugging Face's [`top_k`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_k) argument.
        :param top_p: Tokens comprising the top `top_p` probability mass are kept for sampling. Numbers between 0.0 (exclusive) and 1.0 (inclusive) are allowed. Defaults to 1.0. This is similar to Hugging Face's [`top_p`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_p) argument.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.ServerlessToolAssistedChatCompleteRequest(
            x_friendli_team=x_friendli_team,
            serverless_tool_assisted_chat_complete_body=models.ServerlessToolAssistedChatCompleteBody(
                model=model,
                messages=utils.get_pydantic_model(messages, List[models.Message]),
                eos_token=eos_token,
                frequency_penalty=frequency_penalty,
                max_tokens=max_tokens,
                min_tokens=min_tokens,
                n=n,
                parallel_tool_calls=parallel_tool_calls,
                presence_penalty=presence_penalty,
                repetition_penalty=repetition_penalty,
                response_format=utils.get_pydantic_model(
                    response_format, OptionalNullable[models.ResponseFormat]
                ),
                resume_generation=resume_generation,
                seed=seed,
                stop=stop,
                stream=stream,
                temperature=temperature,
                timeout_microseconds=timeout_microseconds,
                tool_choice=utils.get_pydantic_model(
                    tool_choice,
                    Optional[models.ServerlessToolAssistedChatCompleteBodyToolChoice],
                ),
                tools=utils.get_pydantic_model(
                    tools, OptionalNullable[List[models.ToolAssistedChatTool]]
                ),
                top_k=top_k,
                top_p=top_p,
            ),
        )

        req = self.build_request_async(
            method="POST",
            path="/serverless/tools/v1/chat/completions#complete",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.serverless_tool_assisted_chat_complete_body,
                False,
                False,
                "json",
                models.ServerlessToolAssistedChatCompleteBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="serverlessToolAssistedChatComplete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.ChatResult)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def stream(
        self,
        *,
        model: str,
        messages: Union[List[models.Message], List[models.MessageTypedDict]],
        x_friendli_team: Optional[str] = None,
        eos_token: OptionalNullable[List[int]] = UNSET,
        frequency_penalty: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[int] = UNSET,
        min_tokens: OptionalNullable[int] = 0,
        n: OptionalNullable[int] = 1,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        presence_penalty: OptionalNullable[float] = UNSET,
        repetition_penalty: OptionalNullable[float] = UNSET,
        response_format: OptionalNullable[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = UNSET,
        resume_generation: OptionalNullable[bool] = UNSET,
        seed: OptionalNullable[List[int]] = UNSET,
        stop: OptionalNullable[List[str]] = UNSET,
        stream: OptionalNullable[bool] = True,
        temperature: OptionalNullable[float] = 1,
        timeout_microseconds: OptionalNullable[int] = UNSET,
        tool_choice: Optional[
            Union[
                models.ServerlessToolAssistedChatStreamBodyToolChoice,
                models.ServerlessToolAssistedChatStreamBodyToolChoiceTypedDict,
            ]
        ] = None,
        tools: OptionalNullable[
            Union[
                List[models.ToolAssistedChatTool],
                List[models.ToolAssistedChatToolTypedDict],
            ]
        ] = UNSET,
        top_k: OptionalNullable[int] = 0,
        top_p: OptionalNullable[float] = 1,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Generator[models.StreamedToolAssistedChatResult, None, None]:
        r"""Stream tool assisted chat completions

        Given a list of messages forming a conversation, the model generates a response. Additionally, the model can utilize built-in tools for tool calls, enhancing its capability to provide more comprehensive and actionable responses.

        :param model: Code of the model to use. See [available model list](https://friendli.ai/docs/guides/serverless_endpoints/pricing#text-generation-models).
        :param messages: A list of messages comprising the conversation so far.
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param eos_token: A list of endpoint sentence tokens.
        :param frequency_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled, taking into account their frequency in the preceding text. This penalization diminishes the model's tendency to reproduce identical lines verbatim.
        :param max_tokens: The maximum number of tokens to generate. For decoder-only models like GPT, the length of your input tokens plus `max_tokens` should not exceed the model's maximum length (e.g., 2048 for OpenAI GPT-3). For encoder-decoder models like T5 or BlenderBot, `max_tokens` should not exceed the model's maximum output length. This is similar to Hugging Face's [`max_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.max_new_tokens) argument.
        :param min_tokens: The minimum number of tokens to generate. Default value is 0. This is similar to Hugging Face's [`min_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.min_new_tokens) argument.  **This field is unsupported when `tools` are specified.**
        :param n: The number of independently generated results for the prompt. Not supported when using beam search. Defaults to 1. This is similar to Hugging Face's [`num_return_sequences`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.num_return_sequences) argument.
        :param parallel_tool_calls: Whether to enable parallel function calling.
        :param presence_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled at least once in the existing text.
        :param repetition_penalty: Penalizes tokens that have already appeared in the generated result (plus the input tokens for decoder-only models). Should be greater than or equal to 1.0 (1.0 means no penalty). See [keskar et al., 2019](https://arxiv.org/abs/1909.05858) for more details. This is similar to Hugging Face's [`repetition_penalty`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.repetition_penalty) argument.
        :param response_format: The enforced format of the model's output.  Note that the content of the output message may be truncated if it exceeds the `max_tokens`. You can check this by verifying that the `finish_reason` of the output message is `length`.  ***Important*** You must explicitly instruct the model to produce the desired output format using a system prompt or user message (e.g., `You are an API generating a valid JSON as output.`). Otherwise, the model may result in an unending stream of whitespace or other characters.
        :param resume_generation: Enable to continue text generation even after an error occurs during a tool call.  Note that enabling this option may use more tokens, as the system generates additional content to handle errors gracefully. However, if the system fails more than 8 times, the generation will stop regardless.  ***Tip*** This is useful in scenarios where you want to maintain text generation flow despite errors, such as when generating long-form content. The user will not be interrupted by tool call issues, ensuring a smoother experience.
        :param seed: Seed to control random procedure. If nothing is given, random seed is used for sampling, and return the seed along with the generated result. When using the `n` argument, you can pass a list of seed values to control all of the independent generations.
        :param stop: When one of the stop phrases appears in the generation result, the API will stop generation. The stop phrases are excluded from the result. Defaults to empty list.
        :param stream: Whether to stream generation result. When set true, each token will be sent as [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#event_stream_format) once generated.
        :param temperature: Sampling temperature. Smaller temperature makes the generation result closer to greedy, argmax (i.e., `top_k = 1`) sampling. Defaults to 1.0. This is similar to Hugging Face's [`temperature`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.temperature) argument.
        :param timeout_microseconds: Request timeout. Gives the `HTTP 429 Too Many Requests` response status code. Default behavior is no timeout.
        :param tool_choice: Determines the tool calling behavior of the model. When set to `none`, the model will bypass tool execution and generate a response directly. In `auto` mode (the default), the model dynamically decides whether to call a tool or respond with a message. Alternatively, setting `required` ensures that the model invokes at least one tool before responding to the user. You can also specify a particular tool by `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}`.
        :param tools: A list of tools the model may call. A maximum of 128 functions is supported. Use this to provide a list of functions the model may generate JSON inputs for. For more detailed information about each tool, please refer [here](https://friendli.ai/docs/guides/serverless_endpoints/tools/built_in_tools).  **When `tools` are specified, `min_tokens` field is unsupported.**
        :param top_k: The number of highest probability tokens to keep for sampling. Numbers between 0 and the vocab size of the model (both inclusive) are allowed. The default value is 0, which means that the API does not apply top-k filtering. This is similar to Hugging Face's [`top_k`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_k) argument.
        :param top_p: Tokens comprising the top `top_p` probability mass are kept for sampling. Numbers between 0.0 (exclusive) and 1.0 (inclusive) are allowed. Defaults to 1.0. This is similar to Hugging Face's [`top_p`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_p) argument.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.ServerlessToolAssistedChatStreamRequest(
            x_friendli_team=x_friendli_team,
            serverless_tool_assisted_chat_stream_body=models.ServerlessToolAssistedChatStreamBody(
                model=model,
                messages=utils.get_pydantic_model(messages, List[models.Message]),
                eos_token=eos_token,
                frequency_penalty=frequency_penalty,
                max_tokens=max_tokens,
                min_tokens=min_tokens,
                n=n,
                parallel_tool_calls=parallel_tool_calls,
                presence_penalty=presence_penalty,
                repetition_penalty=repetition_penalty,
                response_format=utils.get_pydantic_model(
                    response_format, OptionalNullable[models.ResponseFormat]
                ),
                resume_generation=resume_generation,
                seed=seed,
                stop=stop,
                stream=stream,
                temperature=temperature,
                timeout_microseconds=timeout_microseconds,
                tool_choice=utils.get_pydantic_model(
                    tool_choice,
                    Optional[models.ServerlessToolAssistedChatStreamBodyToolChoice],
                ),
                tools=utils.get_pydantic_model(
                    tools, OptionalNullable[List[models.ToolAssistedChatTool]]
                ),
                top_k=top_k,
                top_p=top_p,
            ),
        )

        req = self.build_request(
            method="POST",
            path="/serverless/tools/v1/chat/completions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.serverless_tool_assisted_chat_stream_body,
                False,
                False,
                "json",
                models.ServerlessToolAssistedChatStreamBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="serverlessToolAssistedChatStream",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.stream_events(
                http_res,
                lambda raw: utils.unmarshal_json(
                    raw, models.StreamedToolAssistedChatResult
                ),
                sentinel="[DONE]",
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def stream_async(
        self,
        *,
        model: str,
        messages: Union[List[models.Message], List[models.MessageTypedDict]],
        x_friendli_team: Optional[str] = None,
        eos_token: OptionalNullable[List[int]] = UNSET,
        frequency_penalty: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[int] = UNSET,
        min_tokens: OptionalNullable[int] = 0,
        n: OptionalNullable[int] = 1,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        presence_penalty: OptionalNullable[float] = UNSET,
        repetition_penalty: OptionalNullable[float] = UNSET,
        response_format: OptionalNullable[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = UNSET,
        resume_generation: OptionalNullable[bool] = UNSET,
        seed: OptionalNullable[List[int]] = UNSET,
        stop: OptionalNullable[List[str]] = UNSET,
        stream: OptionalNullable[bool] = True,
        temperature: OptionalNullable[float] = 1,
        timeout_microseconds: OptionalNullable[int] = UNSET,
        tool_choice: Optional[
            Union[
                models.ServerlessToolAssistedChatStreamBodyToolChoice,
                models.ServerlessToolAssistedChatStreamBodyToolChoiceTypedDict,
            ]
        ] = None,
        tools: OptionalNullable[
            Union[
                List[models.ToolAssistedChatTool],
                List[models.ToolAssistedChatToolTypedDict],
            ]
        ] = UNSET,
        top_k: OptionalNullable[int] = 0,
        top_p: OptionalNullable[float] = 1,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> AsyncGenerator[models.StreamedToolAssistedChatResult, None]:
        r"""Stream tool assisted chat completions

        Given a list of messages forming a conversation, the model generates a response. Additionally, the model can utilize built-in tools for tool calls, enhancing its capability to provide more comprehensive and actionable responses.

        :param model: Code of the model to use. See [available model list](https://friendli.ai/docs/guides/serverless_endpoints/pricing#text-generation-models).
        :param messages: A list of messages comprising the conversation so far.
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param eos_token: A list of endpoint sentence tokens.
        :param frequency_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled, taking into account their frequency in the preceding text. This penalization diminishes the model's tendency to reproduce identical lines verbatim.
        :param max_tokens: The maximum number of tokens to generate. For decoder-only models like GPT, the length of your input tokens plus `max_tokens` should not exceed the model's maximum length (e.g., 2048 for OpenAI GPT-3). For encoder-decoder models like T5 or BlenderBot, `max_tokens` should not exceed the model's maximum output length. This is similar to Hugging Face's [`max_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.max_new_tokens) argument.
        :param min_tokens: The minimum number of tokens to generate. Default value is 0. This is similar to Hugging Face's [`min_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.min_new_tokens) argument.  **This field is unsupported when `tools` are specified.**
        :param n: The number of independently generated results for the prompt. Not supported when using beam search. Defaults to 1. This is similar to Hugging Face's [`num_return_sequences`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.num_return_sequences) argument.
        :param parallel_tool_calls: Whether to enable parallel function calling.
        :param presence_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled at least once in the existing text.
        :param repetition_penalty: Penalizes tokens that have already appeared in the generated result (plus the input tokens for decoder-only models). Should be greater than or equal to 1.0 (1.0 means no penalty). See [keskar et al., 2019](https://arxiv.org/abs/1909.05858) for more details. This is similar to Hugging Face's [`repetition_penalty`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.repetition_penalty) argument.
        :param response_format: The enforced format of the model's output.  Note that the content of the output message may be truncated if it exceeds the `max_tokens`. You can check this by verifying that the `finish_reason` of the output message is `length`.  ***Important*** You must explicitly instruct the model to produce the desired output format using a system prompt or user message (e.g., `You are an API generating a valid JSON as output.`). Otherwise, the model may result in an unending stream of whitespace or other characters.
        :param resume_generation: Enable to continue text generation even after an error occurs during a tool call.  Note that enabling this option may use more tokens, as the system generates additional content to handle errors gracefully. However, if the system fails more than 8 times, the generation will stop regardless.  ***Tip*** This is useful in scenarios where you want to maintain text generation flow despite errors, such as when generating long-form content. The user will not be interrupted by tool call issues, ensuring a smoother experience.
        :param seed: Seed to control random procedure. If nothing is given, random seed is used for sampling, and return the seed along with the generated result. When using the `n` argument, you can pass a list of seed values to control all of the independent generations.
        :param stop: When one of the stop phrases appears in the generation result, the API will stop generation. The stop phrases are excluded from the result. Defaults to empty list.
        :param stream: Whether to stream generation result. When set true, each token will be sent as [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#event_stream_format) once generated.
        :param temperature: Sampling temperature. Smaller temperature makes the generation result closer to greedy, argmax (i.e., `top_k = 1`) sampling. Defaults to 1.0. This is similar to Hugging Face's [`temperature`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.temperature) argument.
        :param timeout_microseconds: Request timeout. Gives the `HTTP 429 Too Many Requests` response status code. Default behavior is no timeout.
        :param tool_choice: Determines the tool calling behavior of the model. When set to `none`, the model will bypass tool execution and generate a response directly. In `auto` mode (the default), the model dynamically decides whether to call a tool or respond with a message. Alternatively, setting `required` ensures that the model invokes at least one tool before responding to the user. You can also specify a particular tool by `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}`.
        :param tools: A list of tools the model may call. A maximum of 128 functions is supported. Use this to provide a list of functions the model may generate JSON inputs for. For more detailed information about each tool, please refer [here](https://friendli.ai/docs/guides/serverless_endpoints/tools/built_in_tools).  **When `tools` are specified, `min_tokens` field is unsupported.**
        :param top_k: The number of highest probability tokens to keep for sampling. Numbers between 0 and the vocab size of the model (both inclusive) are allowed. The default value is 0, which means that the API does not apply top-k filtering. This is similar to Hugging Face's [`top_k`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_k) argument.
        :param top_p: Tokens comprising the top `top_p` probability mass are kept for sampling. Numbers between 0.0 (exclusive) and 1.0 (inclusive) are allowed. Defaults to 1.0. This is similar to Hugging Face's [`top_p`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_p) argument.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.ServerlessToolAssistedChatStreamRequest(
            x_friendli_team=x_friendli_team,
            serverless_tool_assisted_chat_stream_body=models.ServerlessToolAssistedChatStreamBody(
                model=model,
                messages=utils.get_pydantic_model(messages, List[models.Message]),
                eos_token=eos_token,
                frequency_penalty=frequency_penalty,
                max_tokens=max_tokens,
                min_tokens=min_tokens,
                n=n,
                parallel_tool_calls=parallel_tool_calls,
                presence_penalty=presence_penalty,
                repetition_penalty=repetition_penalty,
                response_format=utils.get_pydantic_model(
                    response_format, OptionalNullable[models.ResponseFormat]
                ),
                resume_generation=resume_generation,
                seed=seed,
                stop=stop,
                stream=stream,
                temperature=temperature,
                timeout_microseconds=timeout_microseconds,
                tool_choice=utils.get_pydantic_model(
                    tool_choice,
                    Optional[models.ServerlessToolAssistedChatStreamBodyToolChoice],
                ),
                tools=utils.get_pydantic_model(
                    tools, OptionalNullable[List[models.ToolAssistedChatTool]]
                ),
                top_k=top_k,
                top_p=top_p,
            ),
        )

        req = self.build_request_async(
            method="POST",
            path="/serverless/tools/v1/chat/completions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.serverless_tool_assisted_chat_stream_body,
                False,
                False,
                "json",
                models.ServerlessToolAssistedChatStreamBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="serverlessToolAssistedChatStream",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.stream_events_async(
                http_res,
                lambda raw: utils.unmarshal_json(
                    raw, models.StreamedToolAssistedChatResult
                ),
                sentinel="[DONE]",
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .streamedtoolassistedchattoken import (
    StreamedToolAssistedChatToken,
    StreamedToolAssistedChatTokenTypedDict,
)
from .streamedtoolassistedchattoolstatus import (
    StreamedToolAssistedChatToolStatus,
    StreamedToolAssistedChatToolStatusTypedDict,
)
from typing import Union


StreamedToolAssistedChatResultTypedDict = Union[
    StreamedToolAssistedChatTokenTypedDict, StreamedToolAssistedChatToolStatusTypedDict
]
r"""A server-sent event containing chat completions content."""


StreamedToolAssistedChatResult = Union[
    StreamedToolAssistedChatToken, StreamedToolAssistedChatToolStatus
]
r"""A server-sent event containing chat completions content."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
import pydantic
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


Type = Literal["text", "json_object", "regex"]
r"""Type of the response format."""


class ResponseFormatTypedDict(TypedDict):
    r"""The enforced format of the model's output.

    Note that the content of the output message may be truncated if it exceeds the `max_tokens`.
    You can check this by verifying that the `finish_reason` of the output message is `length`.

    ***Important***
    You must explicitly instruct the model to produce the desired output format using a system prompt or user message (e.g., `You are an API generating a valid JSON as output.`).
    Otherwise, the model may result in an unending stream of whitespace or other characters.

    """

    type: Type
    r"""Type of the response format."""
    schema_: NotRequired[str]
    r"""The schema of the output. For `{ \"type\": \"json_object\" }`, `schema` should be a serialized string of JSON schema. For `{ \"type\": \"regex\" }`, `schema` should be a regex pattern.

    ***Caveat***
    For the JSON object type, recursive definitions are not supported. Optional properties are also not supported; all properties of `{ \"type\": \"object\" }` are generated regardless of whether they are listed in the `required` field.
    For the regex type, lookaheads/lookbehinds (e.g., `\a`, `\z`, `^`, `$`, `(?=)`, `(?!)`, `(?<=...)`, `(?<!...)`) are not supported. Group specials (e.g., `\w`, `\W`, `\d`, `\D`, `\s`, `\S`) do not support non-ASCII characters. Unicode escape patterns (e.g., `\N`, `\p`, `\P`) are not supported. Additionally, conditional matching (`(?(`) and back-references can cause inefficiency.

    """


class ResponseFormat(BaseModel):
    r"""The enforced format of the model's output.

    Note that the content of the output message may be truncated if it exceeds the `max_tokens`.
    You can check this by verifying that the `finish_reason` of the output message is `length`.

    ***Important***
    You must explicitly instruct the model to produce the desired output format using a system prompt or user message (e.g., `You are an API generating a valid JSON as output.`).
    Otherwise, the model may result in an unending stream of whitespace or other characters.

    """

    type: Type
    r"""Type of the response format."""

    schema_: Annotated[Optional[str], pydantic.Field(alias="schema")] = None
    r"""The schema of the output. For `{ \"type\": \"json_object\" }`, `schema` should be a serialized string of JSON schema. For `{ \"type\": \"regex\" }`, `schema` should be a regex pattern.

    ***Caveat***
    For the JSON object type, recursive definitions are not supported. Optional properties are also not supported; all properties of `{ \"type\": \"object\" }` are generated regardless of whether they are listed in the `required` field.
    For the regex type, lookaheads/lookbehinds (e.g., `\a`, `\z`, `^`, `$`, `(?=)`, `(?!)`, `(?<=...)`, `(?<!...)`) are not supported. Group specials (e.g., `\w`, `\W`, `\d`, `\D`, `\s`, `\S`) do not support non-ASCII characters. Unicode escape patterns (e.g., `\N`, `\p`, `\P`) are not supported. Additionally, conditional matching (`(?(`) and back-references can cause inefficiency.

    """



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from friendli.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


Name = Literal[
    "math:calculator",
    "math:statistics",
    "math:calendar",
    "web:search",
    "web:url",
    "code:python-interpreter",
    "file:text",
    "rag:knowledge-base",
]
r"""The name of the built-in tool."""

Status = Literal["STARTED", "UPDATING", "ENDED", "ERRORED"]
r"""Indicates the current execution status of the tool."""


class StreamedToolAssistedChatToolStatusParametersTypedDict(TypedDict):
    name: str
    r"""The name of the tool’s function parameter."""
    value: str
    r"""The value of the tool’s function parameter."""


class StreamedToolAssistedChatToolStatusParameters(BaseModel):
    name: str
    r"""The name of the tool’s function parameter."""

    value: str
    r"""The value of the tool’s function parameter."""


class FilesTypedDict(TypedDict):
    name: str
    r"""The name of the file generated by the tool’s execution."""
    url: str
    r"""URL of the file generated by the tool’s execution."""


class Files(BaseModel):
    name: str
    r"""The name of the file generated by the tool’s execution."""

    url: str
    r"""URL of the file generated by the tool’s execution."""


class ErrorTypedDict(TypedDict):
    type: str
    r"""The type of error encountered during the tool’s execution."""
    msg: str
    r"""The message of error."""


class Error(BaseModel):
    type: str
    r"""The type of error encountered during the tool’s execution."""

    msg: str
    r"""The message of error."""


class StreamedToolAssistedChatToolStatusDataTypedDict(TypedDict):
    tool_call_id: str
    r"""The ID of the tool call."""
    name: Name
    r"""The name of the built-in tool."""
    status: Status
    r"""Indicates the current execution status of the tool."""
    parameters: List[StreamedToolAssistedChatToolStatusParametersTypedDict]
    timestamp: float
    r"""The Unix timestamp (in seconds) for when the event occurred."""
    result: NotRequired[str]
    r"""The output from the tool’s execution."""
    files: NotRequired[List[FilesTypedDict]]
    message: NotRequired[str]
    r"""Message generated by the tool’s execution."""
    error: NotRequired[ErrorTypedDict]


class StreamedToolAssistedChatToolStatusData(BaseModel):
    tool_call_id: str
    r"""The ID of the tool call."""

    name: Name
    r"""The name of the built-in tool."""

    status: Status
    r"""Indicates the current execution status of the tool."""

    parameters: List[StreamedToolAssistedChatToolStatusParameters]

    timestamp: float
    r"""The Unix timestamp (in seconds) for when the event occurred."""

    result: Optional[str] = None
    r"""The output from the tool’s execution."""

    files: Optional[List[Files]] = None

    message: Optional[str] = None
    r"""Message generated by the tool’s execution."""

    error: Optional[Error] = None


class StreamedToolAssistedChatToolStatusTypedDict(TypedDict):
    data: StreamedToolAssistedChatToolStatusDataTypedDict
    event: Literal["tool_status"]


class StreamedToolAssistedChatToolStatus(BaseModel):
    data: StreamedToolAssistedChatToolStatusData

    EVENT: Annotated[
        Annotated[
            Literal["tool_status"], AfterValidator(validate_const("tool_status"))
        ],
        pydantic.Field(alias="event"),
    ] = "tool_status"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .function import Function, FunctionTypedDict
from friendli.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


FunctionToolType = Literal["function"]
r"""The type of the tool."""


class FunctionToolTypedDict(TypedDict):
    type: FunctionToolType
    r"""The type of the tool."""
    function: FunctionTypedDict


class FunctionTool(BaseModel):
    type: FunctionToolType
    r"""The type of the tool."""

    function: Function



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from friendli.chat import Chat
from friendli.completions import Completions
from friendli.token import Token
from friendli.toolassistedchat import ToolAssistedChat


class Serverless(BaseSDK):
    chat: Chat
    tool_assisted_chat: ToolAssistedChat
    completions: Completions
    token: Token

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.chat = Chat(self.sdk_configuration)
        self.tool_assisted_chat = ToolAssistedChat(self.sdk_configuration)
        self.completions = Completions(self.sdk_configuration)
        self.token = Token(self.sdk_configuration)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .streamedchatchoice import StreamedChatChoice, StreamedChatChoiceTypedDict
from friendli.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class StreamedToolAssistedChatTokenDataTypedDict(TypedDict):
    choices: List[StreamedChatChoiceTypedDict]
    created: int
    r"""The Unix timestamp (in seconds) for when the token sampled."""


class StreamedToolAssistedChatTokenData(BaseModel):
    choices: List[StreamedChatChoice]

    created: int
    r"""The Unix timestamp (in seconds) for when the token sampled."""


class StreamedToolAssistedChatTokenTypedDict(TypedDict):
    data: StreamedToolAssistedChatTokenDataTypedDict


class StreamedToolAssistedChatToken(BaseModel):
    data: StreamedToolAssistedChatTokenData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .function import Function, FunctionTypedDict
from friendli.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ToolType = Literal["function"]
r"""The type of the tool. Currently, only `function` is supported."""


class ToolTypedDict(TypedDict):
    type: ToolType
    r"""The type of the tool. Currently, only `function` is supported."""
    function: FunctionTypedDict


class Tool(BaseModel):
    type: ToolType
    r"""The type of the tool. Currently, only `function` is supported."""

    function: Function



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DetokenizationResultTypedDict(TypedDict):
    r"""Successfully detokenized the tokens."""

    text: NotRequired[str]
    r"""Detokenized text output."""


class DetokenizationResult(BaseModel):
    r"""Successfully detokenized the tokens."""

    text: Optional[str] = None
    r"""Detokenized text output."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ServerlessDetokenizationBodyTypedDict(TypedDict):
    model: NotRequired[str]
    r"""Code of the model to use. See [available model list](https://friendli.ai/docs/guides/serverless_endpoints/pricing#text-generation-models)."""
    tokens: NotRequired[List[int]]
    r"""A token sequence to detokenize."""


class ServerlessDetokenizationBody(BaseModel):
    model: Optional[str] = None
    r"""Code of the model to use. See [available model list](https://friendli.ai/docs/guides/serverless_endpoints/pricing#text-generation-models)."""

    tokens: Optional[List[int]] = None
    r"""A token sequence to detokenize."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .assistantmessage import (
    AssistantMessage,
    AssistantMessageFunction,
    AssistantMessageFunctionTypedDict,
    AssistantMessageRole,
    AssistantMessageType,
    AssistantMessageTypedDict,
    ToolCalls,
    ToolCallsTypedDict,
)
from .chatchoice import (
    ChatChoice,
    ChatChoiceFunction,
    ChatChoiceFunctionTypedDict,
    ChatChoiceMessage,
    ChatChoiceMessageTypedDict,
    ChatChoiceToolCalls,
    ChatChoiceToolCallsTypedDict,
    ChatChoiceType,
    ChatChoiceTypedDict,
)
from .chatresult import ChatResult, ChatResultTypedDict
from .completionsbodywithprompt import (
    CompletionsBodyWithPrompt,
    CompletionsBodyWithPromptTypedDict,
)
from .completionsbodywithtokens import (
    CompletionsBodyWithTokens,
    CompletionsBodyWithTokensTypedDict,
)
from .completionschoice import CompletionsChoice, CompletionsChoiceTypedDict
from .completionsresult import CompletionsResult, CompletionsResultTypedDict
from .dedicatedchatcompletebody import (
    DedicatedChatCompleteBody,
    DedicatedChatCompleteBodyLogitBias,
    DedicatedChatCompleteBodyLogitBiasTypedDict,
    DedicatedChatCompleteBodyStreamOptions,
    DedicatedChatCompleteBodyStreamOptionsTypedDict,
    DedicatedChatCompleteBodyToolChoice,
    DedicatedChatCompleteBodyToolChoiceFunction,
    DedicatedChatCompleteBodyToolChoiceFunctionTypedDict,
    DedicatedChatCompleteBodyToolChoiceObject,
    DedicatedChatCompleteBodyToolChoiceObjectTypedDict,
    DedicatedChatCompleteBodyToolChoiceType,
    DedicatedChatCompleteBodyToolChoiceTypedDict,
    DedicatedChatCompleteBodyTypedDict,
)
from .dedicatedchatcompleteop import (
    DedicatedChatCompleteRequest,
    DedicatedChatCompleteRequestTypedDict,
)
from .dedicatedchatstreambody import (
    DedicatedChatStreamBody,
    DedicatedChatStreamBodyLogitBias,
    DedicatedChatStreamBodyLogitBiasTypedDict,
    DedicatedChatStreamBodyStreamOptions,
    DedicatedChatStreamBodyStreamOptionsTypedDict,
    DedicatedChatStreamBodyToolChoice,
    DedicatedChatStreamBodyToolChoiceFunction,
    DedicatedChatStreamBodyToolChoiceFunctionTypedDict,
    DedicatedChatStreamBodyToolChoiceObject,
    DedicatedChatStreamBodyToolChoiceObjectTypedDict,
    DedicatedChatStreamBodyToolChoiceType,
    DedicatedChatStreamBodyToolChoiceTypedDict,
    DedicatedChatStreamBodyTypedDict,
)
from .dedicatedchatstreamop import (
    DedicatedChatStreamRequest,
    DedicatedChatStreamRequestTypedDict,
)
from .dedicatedcompletionscompletebody import (
    DedicatedCompletionsCompleteBody,
    DedicatedCompletionsCompleteBodyCompletionsBodyWithPrompt,
    DedicatedCompletionsCompleteBodyCompletionsBodyWithPromptTypedDict,
    DedicatedCompletionsCompleteBodyCompletionsBodyWithTokens,
    DedicatedCompletionsCompleteBodyCompletionsBodyWithTokensTypedDict,
    DedicatedCompletionsCompleteBodyTypedDict,
)
from .dedicatedcompletionscompleteop import (
    DedicatedCompletionsCompleteRequest,
    DedicatedCompletionsCompleteRequestTypedDict,
)
from .dedicatedcompletionsstreambody import (
    DedicatedCompletionsStreamBody,
    DedicatedCompletionsStreamBodyCompletionsBodyWithPrompt,
    DedicatedCompletionsStreamBodyCompletionsBodyWithPromptTypedDict,
    DedicatedCompletionsStreamBodyCompletionsBodyWithTokens,
    DedicatedCompletionsStreamBodyCompletionsBodyWithTokensTypedDict,
    DedicatedCompletionsStreamBodyTypedDict,
)
from .dedicatedcompletionsstreamop import (
    DedicatedCompletionsStreamRequest,
    DedicatedCompletionsStreamRequestTypedDict,
)
from .dedicateddetokenizationbody import (
    DedicatedDetokenizationBody,
    DedicatedDetokenizationBodyTypedDict,
)
from .dedicateddetokenizationop import (
    DedicatedDetokenizationRequest,
    DedicatedDetokenizationRequestTypedDict,
)
from .dedicatedtokenizationbody import (
    DedicatedTokenizationBody,
    DedicatedTokenizationBodyTypedDict,
)
from .dedicatedtokenizationop import (
    DedicatedTokenizationRequest,
    DedicatedTokenizationRequestTypedDict,
)
from .detokenizationresult import DetokenizationResult, DetokenizationResultTypedDict
from .filebuiltintool import (
    FileBuiltInTool,
    FileBuiltInToolType,
    FileBuiltInToolTypedDict,
)
from .function import Function, FunctionTypedDict, Parameters, ParametersTypedDict
from .functiontool import FunctionTool, FunctionToolType, FunctionToolTypedDict
from .logprobs import (
    Content,
    ContentTypedDict,
    Logprobs,
    LogprobsTypedDict,
    TopLogprobs,
    TopLogprobsTypedDict,
)
from .message import Message, MessageTypedDict
from .otherbuiltintool import (
    OtherBuiltInTool,
    OtherBuiltInToolType,
    OtherBuiltInToolTypedDict,
)
from .ragbuiltintool import RagBuiltInTool, RagBuiltInToolType, RagBuiltInToolTypedDict
from .responseformat import ResponseFormat, ResponseFormatTypedDict, Type
from .sdkerror import SDKError
from .security import Security, SecurityTypedDict
from .serverlesschatcompletebody import (
    LogitBias,
    LogitBiasTypedDict,
    Object,
    ObjectTypedDict,
    ServerlessChatCompleteBody,
    ServerlessChatCompleteBodyTypedDict,
    StreamOptions,
    StreamOptionsTypedDict,
    ToolChoice,
    ToolChoiceFunction,
    ToolChoiceFunctionTypedDict,
    ToolChoiceType,
    ToolChoiceTypedDict,
)
from .serverlesschatcompleteop import (
    ServerlessChatCompleteRequest,
    ServerlessChatCompleteRequestTypedDict,
)
from .serverlesschatstreambody import (
    ServerlessChatStreamBody,
    ServerlessChatStreamBodyLogitBias,
    ServerlessChatStreamBodyLogitBiasTypedDict,
    ServerlessChatStreamBodyStreamOptions,
    ServerlessChatStreamBodyStreamOptionsTypedDict,
    ServerlessChatStreamBodyToolChoice,
    ServerlessChatStreamBodyToolChoiceFunction,
    ServerlessChatStreamBodyToolChoiceFunctionTypedDict,
    ServerlessChatStreamBodyToolChoiceType,
    ServerlessChatStreamBodyToolChoiceTypedDict,
    ServerlessChatStreamBodyTypedDict,
    ToolChoiceObject,
    ToolChoiceObjectTypedDict,
)
from .serverlesschatstreamop import (
    ServerlessChatStreamRequest,
    ServerlessChatStreamRequestTypedDict,
)
from .serverlesscompletionscompletebody import (
    ServerlessCompletionsCompleteBody,
    ServerlessCompletionsCompleteBodyTypedDict,
)
from .serverlesscompletionscompleteop import (
    ServerlessCompletionsCompleteRequest,
    ServerlessCompletionsCompleteRequestTypedDict,
)
from .serverlesscompletionsstreambody import (
    ServerlessCompletionsStreamBody,
    ServerlessCompletionsStreamBodyCompletionsBodyWithPrompt,
    ServerlessCompletionsStreamBodyCompletionsBodyWithPromptTypedDict,
    ServerlessCompletionsStreamBodyCompletionsBodyWithTokens,
    ServerlessCompletionsStreamBodyCompletionsBodyWithTokensTypedDict,
    ServerlessCompletionsStreamBodyTypedDict,
)
from .serverlesscompletionsstreamop import (
    ServerlessCompletionsStreamRequest,
    ServerlessCompletionsStreamRequestTypedDict,
)
from .serverlessdetokenizationbody import (
    ServerlessDetokenizationBody,
    ServerlessDetokenizationBodyTypedDict,
)
from .serverlessdetokenizationop import (
    ServerlessDetokenizationRequest,
    ServerlessDetokenizationRequestTypedDict,
)
from .serverlesstokenizationbody import (
    ServerlessTokenizationBody,
    ServerlessTokenizationBodyTypedDict,
)
from .serverlesstokenizationop import (
    ServerlessTokenizationRequest,
    ServerlessTokenizationRequestTypedDict,
)
from .serverlesstoolassistedchatcompletebody import (
    ServerlessToolAssistedChatCompleteBody,
    ServerlessToolAssistedChatCompleteBodyToolChoice,
    ServerlessToolAssistedChatCompleteBodyToolChoiceFunction,
    ServerlessToolAssistedChatCompleteBodyToolChoiceFunctionTypedDict,
    ServerlessToolAssistedChatCompleteBodyToolChoiceObject,
    ServerlessToolAssistedChatCompleteBodyToolChoiceObjectTypedDict,
    ServerlessToolAssistedChatCompleteBodyToolChoiceType,
    ServerlessToolAssistedChatCompleteBodyToolChoiceTypedDict,
    ServerlessToolAssistedChatCompleteBodyTypedDict,
)
from .serverlesstoolassistedchatcompleteop import (
    ServerlessToolAssistedChatCompleteRequest,
    ServerlessToolAssistedChatCompleteRequestTypedDict,
)
from .serverlesstoolassistedchatstreambody import (
    ServerlessToolAssistedChatStreamBody,
    ServerlessToolAssistedChatStreamBodyToolChoice,
    ServerlessToolAssistedChatStreamBodyToolChoiceFunction,
    ServerlessToolAssistedChatStreamBodyToolChoiceFunctionTypedDict,
    ServerlessToolAssistedChatStreamBodyToolChoiceObject,
    ServerlessToolAssistedChatStreamBodyToolChoiceObjectTypedDict,
    ServerlessToolAssistedChatStreamBodyToolChoiceType,
    ServerlessToolAssistedChatStreamBodyToolChoiceTypedDict,
    ServerlessToolAssistedChatStreamBodyTypedDict,
)
from .serverlesstoolassistedchatstreamop import (
    ServerlessToolAssistedChatStreamRequest,
    ServerlessToolAssistedChatStreamRequestTypedDict,
)
from .streamedchatchoice import (
    Delta,
    DeltaTypedDict,
    StreamedChatChoice,
    StreamedChatChoiceFunction,
    StreamedChatChoiceFunctionTypedDict,
    StreamedChatChoiceToolCalls,
    StreamedChatChoiceToolCallsTypedDict,
    StreamedChatChoiceType,
    StreamedChatChoiceTypedDict,
)
from .streamedchatresult import (
    Data,
    DataTypedDict,
    StreamedChatResult,
    StreamedChatResultTypedDict,
)
from .streamedcompletionsresult import (
    StreamedCompletionsResult,
    StreamedCompletionsResultData,
    StreamedCompletionsResultDataTypedDict,
    StreamedCompletionsResultTypedDict,
)
from .streamedcompletionstokencomplete import (
    StreamedCompletionsTokenComplete,
    StreamedCompletionsTokenCompleteEvent,
    StreamedCompletionsTokenCompleteTypedDict,
)
from .streamedcompletionstokensampled import (
    Event,
    StreamedCompletionsTokenSampled,
    StreamedCompletionsTokenSampledTypedDict,
)
from .streamedtoolassistedchatresult import (
    StreamedToolAssistedChatResult,
    StreamedToolAssistedChatResultTypedDict,
)
from .streamedtoolassistedchattoken import (
    StreamedToolAssistedChatToken,
    StreamedToolAssistedChatTokenData,
    StreamedToolAssistedChatTokenDataTypedDict,
    StreamedToolAssistedChatTokenTypedDict,
)
from .streamedtoolassistedchattoolstatus import (
    Error,
    ErrorTypedDict,
    Files,
    FilesTypedDict,
    Name,
    Status,
    StreamedToolAssistedChatToolStatus,
    StreamedToolAssistedChatToolStatusData,
    StreamedToolAssistedChatToolStatusDataTypedDict,
    StreamedToolAssistedChatToolStatusParameters,
    StreamedToolAssistedChatToolStatusParametersTypedDict,
    StreamedToolAssistedChatToolStatusTypedDict,
)
from .systemmessage import Role, SystemMessage, SystemMessageTypedDict
from .tokenizationresult import TokenizationResult, TokenizationResultTypedDict
from .tokensequence import TokenSequence, TokenSequenceTypedDict
from .tool import Tool, ToolType, ToolTypedDict
from .toolassistedchattool import ToolAssistedChatTool, ToolAssistedChatToolTypedDict
from .toolmessage import ToolMessage, ToolMessageRole, ToolMessageTypedDict
from .usage import Usage, UsageTypedDict
from .usermessage import UserMessage, UserMessageRole, UserMessageTypedDict

__all__ = [
    "AssistantMessage",
    "AssistantMessageFunction",
    "AssistantMessageFunctionTypedDict",
    "AssistantMessageRole",
    "AssistantMessageType",
    "AssistantMessageTypedDict",
    "ChatChoice",
    "ChatChoiceFunction",
    "ChatChoiceFunctionTypedDict",
    "ChatChoiceMessage",
    "ChatChoiceMessageTypedDict",
    "ChatChoiceToolCalls",
    "ChatChoiceToolCallsTypedDict",
    "ChatChoiceType",
    "ChatChoiceTypedDict",
    "ChatResult",
    "ChatResultTypedDict",
    "CompletionsBodyWithPrompt",
    "CompletionsBodyWithPromptTypedDict",
    "CompletionsBodyWithTokens",
    "CompletionsBodyWithTokensTypedDict",
    "CompletionsChoice",
    "CompletionsChoiceTypedDict",
    "CompletionsResult",
    "CompletionsResultTypedDict",
    "Content",
    "ContentTypedDict",
    "Data",
    "DataTypedDict",
    "DedicatedChatCompleteBody",
    "DedicatedChatCompleteBodyLogitBias",
    "DedicatedChatCompleteBodyLogitBiasTypedDict",
    "DedicatedChatCompleteBodyStreamOptions",
    "DedicatedChatCompleteBodyStreamOptionsTypedDict",
    "DedicatedChatCompleteBodyToolChoice",
    "DedicatedChatCompleteBodyToolChoiceFunction",
    "DedicatedChatCompleteBodyToolChoiceFunctionTypedDict",
    "DedicatedChatCompleteBodyToolChoiceObject",
    "DedicatedChatCompleteBodyToolChoiceObjectTypedDict",
    "DedicatedChatCompleteBodyToolChoiceType",
    "DedicatedChatCompleteBodyToolChoiceTypedDict",
    "DedicatedChatCompleteBodyTypedDict",
    "DedicatedChatCompleteRequest",
    "DedicatedChatCompleteRequestTypedDict",
    "DedicatedChatStreamBody",
    "DedicatedChatStreamBodyLogitBias",
    "DedicatedChatStreamBodyLogitBiasTypedDict",
    "DedicatedChatStreamBodyStreamOptions",
    "DedicatedChatStreamBodyStreamOptionsTypedDict",
    "DedicatedChatStreamBodyToolChoice",
    "DedicatedChatStreamBodyToolChoiceFunction",
    "DedicatedChatStreamBodyToolChoiceFunctionTypedDict",
    "DedicatedChatStreamBodyToolChoiceObject",
    "DedicatedChatStreamBodyToolChoiceObjectTypedDict",
    "DedicatedChatStreamBodyToolChoiceType",
    "DedicatedChatStreamBodyToolChoiceTypedDict",
    "DedicatedChatStreamBodyTypedDict",
    "DedicatedChatStreamRequest",
    "DedicatedChatStreamRequestTypedDict",
    "DedicatedCompletionsCompleteBody",
    "DedicatedCompletionsCompleteBodyCompletionsBodyWithPrompt",
    "DedicatedCompletionsCompleteBodyCompletionsBodyWithPromptTypedDict",
    "DedicatedCompletionsCompleteBodyCompletionsBodyWithTokens",
    "DedicatedCompletionsCompleteBodyCompletionsBodyWithTokensTypedDict",
    "DedicatedCompletionsCompleteBodyTypedDict",
    "DedicatedCompletionsCompleteRequest",
    "DedicatedCompletionsCompleteRequestTypedDict",
    "DedicatedCompletionsStreamBody",
    "DedicatedCompletionsStreamBodyCompletionsBodyWithPrompt",
    "DedicatedCompletionsStreamBodyCompletionsBodyWithPromptTypedDict",
    "DedicatedCompletionsStreamBodyCompletionsBodyWithTokens",
    "DedicatedCompletionsStreamBodyCompletionsBodyWithTokensTypedDict",
    "DedicatedCompletionsStreamBodyTypedDict",
    "DedicatedCompletionsStreamRequest",
    "DedicatedCompletionsStreamRequestTypedDict",
    "DedicatedDetokenizationBody",
    "DedicatedDetokenizationBodyTypedDict",
    "DedicatedDetokenizationRequest",
    "DedicatedDetokenizationRequestTypedDict",
    "DedicatedTokenizationBody",
    "DedicatedTokenizationBodyTypedDict",
    "DedicatedTokenizationRequest",
    "DedicatedTokenizationRequestTypedDict",
    "Delta",
    "DeltaTypedDict",
    "DetokenizationResult",
    "DetokenizationResultTypedDict",
    "Error",
    "ErrorTypedDict",
    "Event",
    "FileBuiltInTool",
    "FileBuiltInToolType",
    "FileBuiltInToolTypedDict",
    "Files",
    "FilesTypedDict",
    "Function",
    "FunctionTool",
    "FunctionToolType",
    "FunctionToolTypedDict",
    "FunctionTypedDict",
    "LogitBias",
    "LogitBiasTypedDict",
    "Logprobs",
    "LogprobsTypedDict",
    "Message",
    "MessageTypedDict",
    "Name",
    "Object",
    "ObjectTypedDict",
    "OtherBuiltInTool",
    "OtherBuiltInToolType",
    "OtherBuiltInToolTypedDict",
    "Parameters",
    "ParametersTypedDict",
    "RagBuiltInTool",
    "RagBuiltInToolType",
    "RagBuiltInToolTypedDict",
    "ResponseFormat",
    "ResponseFormatTypedDict",
    "Role",
    "SDKError",
    "Security",
    "SecurityTypedDict",
    "ServerlessChatCompleteBody",
    "ServerlessChatCompleteBodyTypedDict",
    "ServerlessChatCompleteRequest",
    "ServerlessChatCompleteRequestTypedDict",
    "ServerlessChatStreamBody",
    "ServerlessChatStreamBodyLogitBias",
    "ServerlessChatStreamBodyLogitBiasTypedDict",
    "ServerlessChatStreamBodyStreamOptions",
    "ServerlessChatStreamBodyStreamOptionsTypedDict",
    "ServerlessChatStreamBodyToolChoice",
    "ServerlessChatStreamBodyToolChoiceFunction",
    "ServerlessChatStreamBodyToolChoiceFunctionTypedDict",
    "ServerlessChatStreamBodyToolChoiceType",
    "ServerlessChatStreamBodyToolChoiceTypedDict",
    "ServerlessChatStreamBodyTypedDict",
    "ServerlessChatStreamRequest",
    "ServerlessChatStreamRequestTypedDict",
    "ServerlessCompletionsCompleteBody",
    "ServerlessCompletionsCompleteBodyTypedDict",
    "ServerlessCompletionsCompleteRequest",
    "ServerlessCompletionsCompleteRequestTypedDict",
    "ServerlessCompletionsStreamBody",
    "ServerlessCompletionsStreamBodyCompletionsBodyWithPrompt",
    "ServerlessCompletionsStreamBodyCompletionsBodyWithPromptTypedDict",
    "ServerlessCompletionsStreamBodyCompletionsBodyWithTokens",
    "ServerlessCompletionsStreamBodyCompletionsBodyWithTokensTypedDict",
    "ServerlessCompletionsStreamBodyTypedDict",
    "ServerlessCompletionsStreamRequest",
    "ServerlessCompletionsStreamRequestTypedDict",
    "ServerlessDetokenizationBody",
    "ServerlessDetokenizationBodyTypedDict",
    "ServerlessDetokenizationRequest",
    "ServerlessDetokenizationRequestTypedDict",
    "ServerlessTokenizationBody",
    "ServerlessTokenizationBodyTypedDict",
    "ServerlessTokenizationRequest",
    "ServerlessTokenizationRequestTypedDict",
    "ServerlessToolAssistedChatCompleteBody",
    "ServerlessToolAssistedChatCompleteBodyToolChoice",
    "ServerlessToolAssistedChatCompleteBodyToolChoiceFunction",
    "ServerlessToolAssistedChatCompleteBodyToolChoiceFunctionTypedDict",
    "ServerlessToolAssistedChatCompleteBodyToolChoiceObject",
    "ServerlessToolAssistedChatCompleteBodyToolChoiceObjectTypedDict",
    "ServerlessToolAssistedChatCompleteBodyToolChoiceType",
    "ServerlessToolAssistedChatCompleteBodyToolChoiceTypedDict",
    "ServerlessToolAssistedChatCompleteBodyTypedDict",
    "ServerlessToolAssistedChatCompleteRequest",
    "ServerlessToolAssistedChatCompleteRequestTypedDict",
    "ServerlessToolAssistedChatStreamBody",
    "ServerlessToolAssistedChatStreamBodyToolChoice",
    "ServerlessToolAssistedChatStreamBodyToolChoiceFunction",
    "ServerlessToolAssistedChatStreamBodyToolChoiceFunctionTypedDict",
    "ServerlessToolAssistedChatStreamBodyToolChoiceObject",
    "ServerlessToolAssistedChatStreamBodyToolChoiceObjectTypedDict",
    "ServerlessToolAssistedChatStreamBodyToolChoiceType",
    "ServerlessToolAssistedChatStreamBodyToolChoiceTypedDict",
    "ServerlessToolAssistedChatStreamBodyTypedDict",
    "ServerlessToolAssistedChatStreamRequest",
    "ServerlessToolAssistedChatStreamRequestTypedDict",
    "Status",
    "StreamOptions",
    "StreamOptionsTypedDict",
    "StreamedChatChoice",
    "StreamedChatChoiceFunction",
    "StreamedChatChoiceFunctionTypedDict",
    "StreamedChatChoiceToolCalls",
    "StreamedChatChoiceToolCallsTypedDict",
    "StreamedChatChoiceType",
    "StreamedChatChoiceTypedDict",
    "StreamedChatResult",
    "StreamedChatResultTypedDict",
    "StreamedCompletionsResult",
    "StreamedCompletionsResultData",
    "StreamedCompletionsResultDataTypedDict",
    "StreamedCompletionsResultTypedDict",
    "StreamedCompletionsTokenComplete",
    "StreamedCompletionsTokenCompleteEvent",
    "StreamedCompletionsTokenCompleteTypedDict",
    "StreamedCompletionsTokenSampled",
    "StreamedCompletionsTokenSampledTypedDict",
    "StreamedToolAssistedChatResult",
    "StreamedToolAssistedChatResultTypedDict",
    "StreamedToolAssistedChatToken",
    "StreamedToolAssistedChatTokenData",
    "StreamedToolAssistedChatTokenDataTypedDict",
    "StreamedToolAssistedChatTokenTypedDict",
    "StreamedToolAssistedChatToolStatus",
    "StreamedToolAssistedChatToolStatusData",
    "StreamedToolAssistedChatToolStatusDataTypedDict",
    "StreamedToolAssistedChatToolStatusParameters",
    "StreamedToolAssistedChatToolStatusParametersTypedDict",
    "StreamedToolAssistedChatToolStatusTypedDict",
    "SystemMessage",
    "SystemMessageTypedDict",
    "TokenSequence",
    "TokenSequenceTypedDict",
    "TokenizationResult",
    "TokenizationResultTypedDict",
    "Tool",
    "ToolAssistedChatTool",
    "ToolAssistedChatToolTypedDict",
    "ToolCalls",
    "ToolCallsTypedDict",
    "ToolChoice",
    "ToolChoiceFunction",
    "ToolChoiceFunctionTypedDict",
    "ToolChoiceObject",
    "ToolChoiceObjectTypedDict",
    "ToolChoiceType",
    "ToolChoiceTypedDict",
    "ToolMessage",
    "ToolMessageRole",
    "ToolMessageTypedDict",
    "ToolType",
    "ToolTypedDict",
    "TopLogprobs",
    "TopLogprobsTypedDict",
    "Type",
    "Usage",
    "UsageTypedDict",
    "UserMessage",
    "UserMessageRole",
    "UserMessageTypedDict",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from friendli.friendli_chat import FriendliChat
from friendli.friendli_completions import FriendliCompletions
from friendli.friendli_token import FriendliToken


class Dedicated(BaseSDK):
    chat: FriendliChat
    completions: FriendliCompletions
    token: FriendliToken

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.chat = FriendliChat(self.sdk_configuration)
        self.completions = FriendliCompletions(self.sdk_configuration)
        self.token = FriendliToken(self.sdk_configuration)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .streamedchatchoice import StreamedChatChoice, StreamedChatChoiceTypedDict
from .usage import Usage, UsageTypedDict
from friendli.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DataTypedDict(TypedDict):
    choices: List[StreamedChatChoiceTypedDict]
    created: int
    r"""The Unix timestamp (in seconds) for when the token sampled."""
    usage: NotRequired[UsageTypedDict]


class Data(BaseModel):
    choices: List[StreamedChatChoice]

    created: int
    r"""The Unix timestamp (in seconds) for when the token sampled."""

    usage: Optional[Usage] = None


class StreamedChatResultTypedDict(TypedDict):
    r"""A server-sent event containing chat completions content."""

    data: DataTypedDict


class StreamedChatResult(BaseModel):
    r"""A server-sent event containing chat completions content."""

    data: Data



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .logprobs import Logprobs, LogprobsTypedDict
from friendli.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


ChatChoiceType = Literal["function"]
r"""The type of the tool."""


class ChatChoiceFunctionTypedDict(TypedDict):
    name: str
    r"""The name of the function to call."""
    arguments: str
    r"""The arguments for calling the function, generated by the model in JSON format.
    Ensure to validate these arguments in your code before invoking the function since the model may not always produce valid JSON.

    """


class ChatChoiceFunction(BaseModel):
    name: str
    r"""The name of the function to call."""

    arguments: str
    r"""The arguments for calling the function, generated by the model in JSON format.
    Ensure to validate these arguments in your code before invoking the function since the model may not always produce valid JSON.

    """


class ChatChoiceToolCallsTypedDict(TypedDict):
    id: str
    r"""The ID of the tool call."""
    type: ChatChoiceType
    r"""The type of the tool."""
    function: ChatChoiceFunctionTypedDict


class ChatChoiceToolCalls(BaseModel):
    id: str
    r"""The ID of the tool call."""

    type: ChatChoiceType
    r"""The type of the tool."""

    function: ChatChoiceFunction


class ChatChoiceMessageTypedDict(TypedDict):
    role: str
    r"""Role of the generated message author, in this case `assistant`."""
    content: NotRequired[str]
    r"""The contents of the assistant message."""
    tool_calls: NotRequired[List[ChatChoiceToolCallsTypedDict]]


class ChatChoiceMessage(BaseModel):
    role: str
    r"""Role of the generated message author, in this case `assistant`."""

    content: Optional[str] = None
    r"""The contents of the assistant message."""

    tool_calls: Optional[List[ChatChoiceToolCalls]] = None


class ChatChoiceTypedDict(TypedDict):
    index: int
    r"""The index of the choice in the list of generated choices."""
    message: ChatChoiceMessageTypedDict
    finish_reason: str
    r"""Termination condition of the generation. `stop` means the API returned the full chat completions generated by the model without running into any limits.
    `length` means the generation exceeded `max_tokens` or the conversation exceeded the max context length.
    `tool_calls` means the API has generated tool calls.

    """
    logprobs: NotRequired[LogprobsTypedDict]
    r"""Log probability information for the choice."""


class ChatChoice(BaseModel):
    index: int
    r"""The index of the choice in the list of generated choices."""

    message: ChatChoiceMessage

    finish_reason: str
    r"""Termination condition of the generation. `stop` means the API returned the full chat completions generated by the model without running into any limits.
    `length` means the generation exceeded `max_tokens` or the conversation exceeded the max context length.
    `tool_calls` means the API has generated tool calls.

    """

    logprobs: Optional[Logprobs] = None
    r"""Log probability information for the choice."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


ToolMessageRole = Literal["tool"]
r"""The role of the messages author."""


class ToolMessageTypedDict(TypedDict):
    role: ToolMessageRole
    r"""The role of the messages author."""
    content: str
    r"""The content of tool message that contains the result of tool calling."""
    tool_call_id: str
    r"""The ID of tool call corresponding to this message."""
    name: NotRequired[str]
    r"""An optional name of the tool call corresponding to this message."""


class ToolMessage(BaseModel):
    role: ToolMessageRole
    r"""The role of the messages author."""

    content: str
    r"""The content of tool message that contains the result of tool calling."""

    tool_call_id: str
    r"""The ID of tool call corresponding to this message."""

    name: Optional[str] = None
    r"""An optional name of the tool call corresponding to this message."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .completionsbodywithprompt import (
    CompletionsBodyWithPrompt,
    CompletionsBodyWithPromptTypedDict,
)
from .completionsbodywithtokens import (
    CompletionsBodyWithTokens,
    CompletionsBodyWithTokensTypedDict,
)
from typing import Union


ServerlessCompletionsCompleteBodyTypedDict = Union[
    CompletionsBodyWithPromptTypedDict, CompletionsBodyWithTokensTypedDict
]


ServerlessCompletionsCompleteBody = Union[
    CompletionsBodyWithPrompt, CompletionsBodyWithTokens
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .serverlesstoolassistedchatstreambody import (
    ServerlessToolAssistedChatStreamBody,
    ServerlessToolAssistedChatStreamBodyTypedDict,
)
from friendli.types import BaseModel
from friendli.utils import FieldMetadata, HeaderMetadata, RequestMetadata
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ServerlessToolAssistedChatStreamRequestTypedDict(TypedDict):
    serverless_tool_assisted_chat_stream_body: (
        ServerlessToolAssistedChatStreamBodyTypedDict
    )
    x_friendli_team: NotRequired[str]
    r"""ID of team to run requests as (optional parameter)."""


class ServerlessToolAssistedChatStreamRequest(BaseModel):
    serverless_tool_assisted_chat_stream_body: Annotated[
        ServerlessToolAssistedChatStreamBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_friendli_team: Annotated[
        Optional[str],
        pydantic.Field(alias="X-Friendli-Team"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""ID of team to run requests as (optional parameter)."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class CompletionsChoiceTypedDict(TypedDict):
    index: int
    r"""The index of the choice in the list of generated choices."""
    seed: int
    r"""Random seed used for the generation."""
    text: str
    r"""Generated text output."""
    tokens: List[int]
    r"""Generated output tokens."""


class CompletionsChoice(BaseModel):
    index: int
    r"""The index of the choice in the list of generated choices."""

    seed: int
    r"""Random seed used for the generation."""

    text: str
    r"""Generated text output."""

    tokens: List[int]
    r"""Generated output tokens."""
